Follow Me
=========
Follow Me provides an API for keeping track of what views, taxonomy terms, etc.
a user has viewed leading up to viewing the current page.

This can be especially helpful on sites that provide multiple ways to get to the
same content. For example, each piece of content might be associated with
several different taxonomy terms, and you need to provide a way for the site
visitor to get back from content to the right term.

Consider these e-commerce use cases:

- Site visitors can drill-down into categories to see what items are there. Each
  item can be in multiple categories. Product information might be spread out
  across several pages. You need a way for the site visitor to easily return to
  the category they selected prior to opening the product page.

- Site visitors can perform searches for products. Once on a product page, you
  want to give site visitors a way to get from one of the search results to the
  next one, without having to go back to the search result page.

This module provides an API to help with these use cases. At present, it does
not provide any user-facing functionality (for example, it does not provide
breadcrumbs) to help with the use cases, but may provide such functionality in
the future. You could, however, use the API exposed by this module to roll your
own breadcrumbs functionality.

## How It Works
Once the top-level `follow_me` module has been enabled, this module will
automatically keep track of the "trail" that a user follows to get to content by
adding a `from_trail` query string parameter to page links. As a user navigates
through the site, the module keeps a "trail component" stack in cache bins,
representing the path that the user has taken so far. This is preferred to using
the session, as it allows site visitors to have multiple tabs open without
bleeding context from one to the other.

The API automatically prunes the stack whenever:
- the user arrives at the site homepage
- a taxonomy term in the same vocabulary as an earlier taxonomy term in the
  stack (if `fm_taxonomies` is enabled).
- the same view as earlier in the stack (if `fm_views` is enabled).

## Definitions
When a user navigates from a page containing a view or taxonomy term to content,
that view or taxonomy term is considered the **parent object** for the content.

In addition, each such view or taxonomy term that the user navigates through is
referred to as a **trail component**.

Within each component is the **context** necessary to obtain a reference to the
view or term.

## Limitations and Drawbacks
Although useful, this module does have several drawbacks:

### Accuracy
Pages that contain multiple views or taxonomy terms that lead to content may
pose a challenge. At present, this module only captures at most one of each when
capturing the context for a trail component. The links generated by this module
also do not provide enough context to determine which of several parent objects
are being used to navigate to the content.

For example, if:
1. A page has two views.
2. The same node appears in both views.
3. The user clicks a link in the first view.

This module may not currently properly identify that the first view is the
parent of the node.

### Performance
This module works by pre-processing and modifying links site-wide. Although the
module explicitly avoids altering links in the admin interface, any module which
hooks into `theme_link()` necessarily comes with a performance penalty.

### Usability
In general, keeping track of where a user came from before getting to the
current page is not normally as helpful to the user experience as indicating to
the user where they are in the site structure or guiding the user to other
content of interest.

See, [The Problem with Breadcrumb Trails](https://derivadow.com/2010/02/18/the-problem-with-breadcrumb-trails/)
by Michael Smethurst for a great write-up on this topic.

## Sub-modules
- "Follow Me, Views" (`fm_views`) integrates Follow Me with Views, allowing FM
  to capture context information about what views the logged-in user visits
  on the way to content.

- "Follow Me, Taxonomies" (`fm_taxonomies`) integrates Follow Me with Taxonomy,
  allowing FM to capture context information about what taxonomy terms the
  logged-in user visits on the way to content.

There is currently no sub-module for capturing what nodes the user visits. This
is currently by design, as all of our current use cases for this module have
only needed us to track the _collections of content_ (views and taxonomies) the
user visited to get to content rather than tracking the content itself.

If node-based context functionality is needed,
copying and lightly editing what was done for `fm_taxonomies` as a template
should be a good start. Patches are welcome.

## Using the API
This section details several common use cases for the API.

### Getting the Parent View / Taxonomy Term of the Current Page
The `follow_me_get_parent()` function can be used to obtain each of the parent
objects for the current page. You can also pass in an object type to get a
specific parent object type, if one exists. For example, to get the view that
lead to the current page:

    $parent_view = follow_me_get_parent('view');

In addition, if that initial page contains both a view _and_ a taxonomy term
(for example, because you're using
[Taxonomy Display](https://www.drupal.org/project/taxonomy_display)  module),
and both `fm_taxonomies` and `fm_views` are enabled, the context for that page
will include both the taxonomy term and the view. You can get all of the
parent objects by calling `follow_me_get_parent()` without any arguments.

You can also request each of the parent objects in isolation, as follows:

    $parent_view = follow_me_get_parent('view');
    $parent_term = follow_me_get_parent('taxonomy_term');

By default, `follow_me_get_parent()` only searches the trail component that
immediately preceded the current page. To search further back, use the
`$depth_limit` parameter, as follows:

    // Search up to five components back in the trail
    $parent_view = follow_me_get_parent('view', 5);

The trail is organized like a LIFO stack, which means that the
most-recently-viewed page is first in the trail.

**NOTE:** There is no guarantee that there is any parent object at all for the
current page, even if a trail ID has been passed-in on the query string.

Trail information is stored in cache tables, by design, and should be considered
temporary / ephemeral. It is feasible that a user could bookmark a link that
references a trail that no longer exists when the user returns to the link in
the future. Be sure to have some default logic in place when it is not possible
to determine the parent object of the current page.

### Getting the Current Trail ID
The trail ID for the current page can be obtained using
`follow_me_get_current_trail_id()`. If the current page is not part of a trail,
a new trail containing the page will be automatically generated the first time
this function is called, unless the `$read_only` parameter is passed as `TRUE`.

All links on every non-admin page automatically invoke this function. Unless you
are taking direct control over link generation, you should not normally need to
add the trail ID to a link.

Bear in mind that the _current_ trail ID is not the same as the ID that may
have been passed-in on the query string; that ID is referred to as the
_previous_ trail ID. The _current_ trail ID is the ID of the trail that
terminates at the current page, while the trail identified by the _previous_
trail ID terminates at the previous page.

### Generating Trail IDs Manually
In some cases, it may be useful to "spoof" a trail or control which trail
component is the terminal component for a trail. For example, it may be the case
that you would like to generate a trail for a view that is invoked
programmatically under the hood.

Here's how you would do that:

    $view = views_get_view('my_view');

    // Get page 5 of results (they're zero-indexed in Drupal 7)
    $page_number = 4;

    $view->set_arguments(array('arg1', 'arg2');
    $view->set_current_page($page_number);
    $view->execute();

    $context   = fm_views_serialize_to_context($view);
    $component = follow_me_get_as_trail_component($context);

    // Ensure the context includes the page number
    // This is a workaround for a current limitation in how Follow Me captures
    // view context.
    $component['query']['page'] = $page_number;

    $trail_id = follow_me_generate_trail_id($component);

The trail ID in `$trail_id` can then be added to the query string for any link
to a page that should simulate being the child of page 4 of results in the
`my_view` view.
